import { Validators } from '@angular/forms';
import { parseDate } from '../utils';
export function isPresent(obj) {
    return obj !== undefined && obj !== null;
}
function isEmptyInputValue(value) {
    if (value && typeof value === 'string') {
        return !value.trim();
    }
    return value == null || value.length === 0;
}
function isNumber(value) {
    if (typeof value === 'number') {
        return true;
    }
    if (typeof value === 'string') {
        var parsedValue = parseInt(value, 10);
        if (value.length === parsedValue.toString().length && Number.isInteger(parsedValue)) {
            return true;
        }
    }
    return false;
}
export function requiredWithTrim(control) {
    return isEmptyInputValue(control.value) ? { 'requiredWithTrim': true } : null;
}
export function rangeLengthValidator(rangeLength) {
    return function (control) {
        if (isPresent(Validators.required(control))) {
            return null;
        }
        var value = control.value;
        var min = rangeLength[0] || 0;
        var max = rangeLength[1] || Number.MAX_VALUE;
        return value.length >= min && value.length <= max ? null : { 'hiRangeLength': true };
    };
}
export function minValidator(min) {
    return function (control) {
        if (isPresent(Validators.required(control))) {
            return null;
        }
        var v = control.value;
        return v >= min ? null : { 'hiMin': true };
    };
}
export function maxValidator(max) {
    return function (control) {
        if (isPresent(Validators.required(control))) {
            return null;
        }
        var v = control.value;
        return v <= max ? null : { 'hiMax': true };
    };
}
export function rangeValidator(range) {
    return function (control) {
        if (isPresent(Validators.required(control))) {
            return null;
        }
        var v = control.value;
        var min = range[0] || Number.MIN_VALUE;
        var max = range[1] || Number.MAX_VALUE;
        return v >= min && v <= max ? null : { 'hiRange': true };
    };
}
export function digitsValidator() {
    return function (control) {
        if (isPresent(Validators.required(control))) {
            return null;
        }
        var v = control.value;
        return /^\d+$/.test(v) ? null : { 'hiDigits': true };
    };
}
export function numberValidator() {
    return function (control) {
        if (isPresent(Validators.required(control))) {
            return null;
        }
        var v = control.value;
        return /^(?:-?\d+|-?\d{1,3}(?:,\d{3})+)?(?:\.\d+)?$/.test(v) ? null : { 'hiNumber': true };
    };
}
export function urlValidator() {
    return function (control) {
        if (isPresent(Validators.required(control))) {
            return null;
        }
        var v = control.value;
        return /^(?:(?:(?:https?|ftp):)?\/\/)(?:\S+(?::\S*)?@)?(?:(?!(?:10|127)(?:\.\d{1,3}){3})(?!(?:169\.254|192\.168)(?:\.\d{1,3}){2})(?!172\.(?:1[6-9]|2\d|3[0-1])(?:\.\d{1,3}){2})(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5])){2}(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4]))|(?:(?:[a-z\u00a1-\uffff0-9]-*)*[a-z\u00a1-\uffff0-9]+)(?:\.(?:[a-z\u00a1-\uffff0-9]-*)*[a-z\u00a1-\uffff0-9]+)*(?:\.(?:[a-z\u00a1-\uffff]{2,})).?)(?::\d{2,5})?(?:[/?#]\S*)?$/i.test(v) ? null : { 'hiUrl': true };
    };
}
export function emailValidator() {
    return function (control) {
        if (isPresent(Validators.required(control))) {
            return null;
        }
        var v = control.value;
        return /^[a-zA-Z0-9.!#$%&'*+\/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?)*$/.test(v) ? null : { 'hiEmail': true };
    };
}
export function dateValidator() {
    return function (control) {
        if (isPresent(Validators.required(control))) {
            return null;
        }
        var v = control.value;
        return !/Invalid|NaN/.test(new Date(v).toString()) ? null : { 'hiDate': true };
    };
}
export function dateISOValidator() {
    return function (control) {
        if (isPresent(Validators.required(control))) {
            return null;
        }
        var v = control.value;
        return /^\d{4}[\/\-](0?[1-9]|1[012])[\/\-](0?[1-9]|[12][0-9]|3[01])$/.test(v) ? null : { 'hiDateISO': true };
    };
}
export function jsonValidator() {
    return function (control) {
        if (isPresent(Validators.required(control))) {
            return null;
        }
        var value = control.value;
        try {
            var obj = JSON.parse(value);
            if (Boolean(obj) && typeof obj === 'object') {
                return null;
            }
            return { 'hiJson': true };
        }
        catch (e) {
            return { 'hiJson': true };
        }
    };
}
export function base64Validator() {
    return function (control) {
        if (isPresent(Validators.required(control))) {
            return null;
        }
        var v = control.value;
        return /^(data:image\/\w+;base64,)?(?:[A-Z0-9+\/]{4})*(?:[A-Z0-9+\/]{2}==|[A-Z0-9+\/]{3}=|[A-Z0-9+\/]{4})$/i.test(v) ? null : { 'hiBase64': true };
    };
}
export function phoneValidator(locale) {
    var phones = {
        'zh-CN': /^(\+?0?86\-?)?((13\d|14[57]|15[^4,\D]|17[678]|18\d)\d{8}|170[059]\d{7})$/,
        'zh-TW': /^(\+?886\-?|0)?9\d{8}$/,
        'en-ZA': /^(\+?27|0)\d{9}$/,
        'en-AU': /^(\+?61|0)4\d{8}$/,
        'en-HK': /^(\+?852\-?)?[569]\d{3}\-?\d{4}$/,
        'fr-FR': /^(\+?33|0)[67]\d{8}$/,
        'de-DE': /^(\+?49|0)[1-9]\d{10}$/,
        'pt-PT': /^(\+351)?9[1236]\d{7}$/,
        'el-GR': /^(\+?30)?(69\d{8})$/,
        'en-GB': /^(\+?44|0)7\d{9}$/,
        'en-US': /^(\+?1)?[2-9]\d{2}[2-9](?!11)\d{6}$/,
        'en-ZM': /^(\+26)?09[567]\d{7}$/,
        'ru-RU': /^(\+?7|8)?9\d{9}$/,
        'nb-NO': /^(\+?47)?[49]\d{7}$/,
        'nn-NO': /^(\+?47)?[49]\d{7}$/,
        'vi-VN': /^(0|\+?84)?((1(2([0-9])|6([2-9])|88|99))|(9((?!5)[0-9])))([0-9]{7})$/,
        'en-NZ': /^(\+?64|0)2\d{7,9}$/,
        'hu-HU': /^(?:\+?(?:36|\(36\)))[ -\/]?(?:(?:(?:(?!1|20|21|30|31|70|90)[2-9][0-9])[ -\/]?\d{3}[ -\/]?\d{3})|(?:(?:1|20|21|30|31|70|90)[ -\/]?\d{3}[ -\/]?\d{2}[ -\/]?\d{2}))$/,
        'nl-NL': /^(^\+[0-9]{2}|^\+[0-9]{2}\(0\)|^\(\+[0-9]{2}\)\(0\)|^00[0-9]{2}|^0)([0-9]{9}$|[0-9\-\s]{10}$)$/
    };
    return function (control) {
        if (isPresent(Validators.required(control))) {
            return null;
        }
        var v = control.value;
        var pattern = phones[locale] || phones['en-US'];
        return (new RegExp(pattern)).test(v) ? null : { 'hiPhone': true };
    };
}
export function uuidValidator(version) {
    var uuid = {
        '3': /^[0-9A-F]{8}-[0-9A-F]{4}-3[0-9A-F]{3}-[0-9A-F]{4}-[0-9A-F]{12}$/i,
        '4': /^[0-9A-F]{8}-[0-9A-F]{4}-4[0-9A-F]{3}-[89AB][0-9A-F]{3}-[0-9A-F]{12}$/i,
        '5': /^[0-9A-F]{8}-[0-9A-F]{4}-5[0-9A-F]{3}-[89AB][0-9A-F]{3}-[0-9A-F]{12}$/i,
        'all': /^[0-9A-F]{8}-[0-9A-F]{4}-[0-9A-F]{4}-[0-9A-F]{4}-[0-9A-F]{12}$/i
    };
    return function (control) {
        if (isPresent(Validators.required(control))) {
            return null;
        }
        var v = control.value;
        var pattern = uuid[version] || uuid.all;
        return (new RegExp(pattern)).test(v) ? null : { 'hiUUID': true };
    };
}
export function equalValidator(val) {
    return function (control) {
        if (isPresent(Validators.required(control))) {
            return null;
        }
        var v = control.value;
        return val === v ? null : { hiEqual: true };
    };
}
export function equalToValidator(target) {
    var subscription;
    return function (control) {
        if (isPresent(Validators.required(control))) {
            return null;
        }
        var targetControl = typeof target === 'string' ? control.root.get(target) : target;
        if (!subscription) {
            subscription = targetControl.valueChanges.subscribe(function () { return control.updateValueAndValidity(); });
        }
        var sourceValue = control.value;
        var targetValue = targetControl.value;
        return sourceValue === targetValue ? null : { hiEqualTo: true };
    };
}
export function includesValidator(optionalValue) {
    return function (control) {
        if (isPresent(Validators.required(control))) {
            return null;
        }
        return (optionalValue || []).indexOf((control.value || '').trim()) !== -1 ? null : { 'hiIncludes': true };
    };
}
export function beforeDateValidator(target) {
    return function (control) {
        if (isPresent(Validators.required(control))) {
            return null;
        }
        if (!target) {
            target = new Date();
        }
        if (isNumber(target)) {
            target = parseInt(target, 10);
        }
        var date = parseDate(target);
        var targetValue = control.value;
        if (isNumber(targetValue)) {
            targetValue = parseInt(targetValue, 10);
        }
        var targetDate = parseDate(targetValue);
        return date.getTime() >= targetDate.getTime() ? null : { 'hiBeforeDate': true };
    };
}
export function afterDateValidator(target) {
    return function (control) {
        if (isPresent(Validators.required(control))) {
            return null;
        }
        if (!target) {
            target = new Date();
        }
        if (isNumber(target)) {
            target = parseInt(target, 10);
        }
        var date = parseDate(target);
        var targetValue = control.value;
        if (isNumber(targetValue)) {
            targetValue = parseInt(targetValue, 10);
        }
        var targetDate = parseDate(targetValue);
        return date.getTime() <= targetDate.getTime() ? null : { 'hiAfterDate': true };
    };
}
